#include <wxnc/boxsizer.hpp>
#include <wxnc/sizer.hpp>
#include <wxnc/window.hpp>
#include <wx/gdicmn.h>
#include "internal.h"

wxncPackable::wxncPackable(const wxPoint &pos, const wxSize &size) :
	m_minsize(wxDefaultSize),
	m_size(size),
	m_clientsize(size),
	m_pos(pos),
	m_clientpos(pos)
{
}

wxncPackable::~wxncPackable(void)
{
}

int wxncPackable::GetMinWidth(void) const
{
	return GetMinSize().x;
}

int wxncPackable::GetMinHeight(void) const
{
	return GetMinSize().y;
}

void wxncPackable::GetMinSize(int *w, int *h) const
{
	*w = m_minsize.x;
	*h = m_minsize.y;
}

wxSize wxncPackable::GetMinSize(void) const
{
	return m_minsize;
}

wxPoint wxncPackable::GetPosition(void) const
{
	return m_pos;
}

void wxncPackable::SetPosition(int x, int y)
{
	m_clientpos = m_pos = wxPoint(x, y);
}

void wxncPackable::SetPosition(const wxPoint &pos)
{
	m_clientpos = m_pos = pos;
}

wxRect wxncPackable::GetClientRect(void) const /* unused */
{
	return wxRect(m_clientpos.x, m_clientpos.y,
	       m_clientsize.x, m_clientsize.y);
}

wxSize wxncPackable::GetClientSize(void) const /* unused */
{
	return m_clientsize;
}

int wxncPackable::GetWidth(void) const
{
	return m_size.x;
}

int wxncPackable::GetHeight(void) const
{
	return m_size.y;
}

wxSize wxncPackable::GetSize(void) const
{
	return m_size;
}

void wxncPackable::SetSize(const wxSize &size)
{
	m_clientsize = m_size = size;
}

void wxncPackable::SetSize(int x, int y)
{
	SetSize(wxSize(x, y));
}

wxncSizer::wxncSizer(void)
{
}

wxncSizer::~wxncSizer(void)
{
}

void wxncSizer::Add(wxncPackable *p, int proportion, int flag, int border)
{
	window_list.push_back(p);
}

void wxncSizer::SetSizeHints(wxncWindow *)
{
}

wxncBoxSizer::wxncBoxSizer(int orientation)
{
	m_orient = orientation;
}

void wxncBoxSizer::RecalcSizes(void)
{
	int tx = 0, ty = 0, cx, cy;

	for (std::list<wxncPackable *>::const_iterator w = window_list.begin();
	     w != window_list.end(); ++w)
	{
		(*w)->RecalcSizes();
		(*w)->GetMinSize(&cx, &cy);
		if (m_orient == wxHORIZONTAL) {
			tx += cx;
			if (cy > ty)
				ty = cy;
		} else {
			ty += cy;
			if (cx > tx)
				tx = cx;
		}
	}

	m_minsize = wxSize(tx, ty);
}

void wxncBoxSizer::Layout(void)
{
	int x = m_pos.x, y = m_pos.y;

	for (std::list<wxncPackable *>::const_iterator
	     p = window_list.begin(); p != window_list.end(); ++p)
	{
		(*p)->SetPosition(x, y);
		(*p)->SetSize((*p)->GetMinSize());
		if (m_orient == wxHORIZONTAL)
			x += (*p)->GetMinWidth();
		else
			y += (*p)->GetMinHeight();
		(*p)->Layout();
	}

	m_size = m_minsize;
}
