#include <list>
#include <typeinfo>
#include <wxnc/boxsizer.hpp>
#include <wxnc/color.hpp>
#include <wxnc/toplevel.hpp>
#include <wxnc/window.hpp>
#include <wx/defs.h>
#include <wx/gdicmn.h>
#include <wx/toplevel.h>
#include <ncurses.h>
#include "internal.h"

wxncTopLevelWindow::wxncTopLevelWindow(wxncWindow *parent, wxWindowID id,
    const wxString &title, const wxPoint &pos, const wxSize &size,
    long style, const wxString &name) :
	wxncWindow(parent, id, pos, size, style, name)
{
	m_toplevelwd_flags = style;
	m_title = title;
}

wxncTopLevelWindow::~wxncTopLevelWindow(void)
{
}

void wxncTopLevelWindow::SetTitle(const wxString &title)
{
	m_title = title;
}

void wxncTopLevelWindow::render_border(int sx, int sy, int ex, int ey)
{
	wxncWindow::render_border(sx, sy, ex, ey);

	int dx = ex - sx - 2;
	int symbols_left_pos = sx + 1;
	int symbols_right_pos = ex - 1;

	attrset(wxncColor::GetColor(wxncColor::
		b_coherence_map[m_bgcolor.nc_index()][2], m_bgcolor));

	if ((m_toplevelwd_flags & wxSYSTEM_MENU) && dx >= 3) {
		mvaddstr(sy, symbols_left_pos, "[*]");
		dx -= 3;
		symbols_left_pos += 3;
	}
	if ((m_toplevelwd_flags & wxCLOSE_BOX) && dx >= 3) {
		symbols_right_pos -= 3;
		dx -= 3;
		mvaddstr(sy, symbols_right_pos, "[/]");
	}
	if ((m_toplevelwd_flags & wxMAXIMIZE_BOX) && dx >= 3) {
		symbols_right_pos -= 3;
		dx -= 3;
		mvaddstr(sy, symbols_right_pos, "[O]");
	}
	if ((m_toplevelwd_flags & wxMINIMIZE_BOX) && dx >= 3) {
		symbols_right_pos -= 3;
		dx -= 3;
		mvaddstr(sy, symbols_right_pos, "[_]");
	}

	if ((m_toplevelwd_flags & wxCAPTION) &&
	    dx >= 6 + static_cast<int>(m_title.Length())) {
		int c = (symbols_left_pos + symbols_right_pos -
		        m_title.Length() - 6) / 2;

		attrset(wxncColor::GetColor(wxncColor::
			b_coherence_map[m_bgcolor.nc_index()][3], m_bgcolor));
		mvaddch(sy, ++c, '[');
		mvaddch(sy, ++c, ' ');
		mvaddstr(sy, ++c, m_title.ToUTF8());
		c += m_title.Length();
		mvaddch(sy, c, ' ');
		mvaddch(sy, ++c, ']');
	}
}
wxncWindow::wxncWindow(wxncWindow *parent, wxWindowID id, const wxPoint &pos,
    const wxSize &size, long style, const wxString &name) :
	wxncPackable(pos, size),
	m_bgcolor(wxncGRAY), m_fgcolor(wxncBLACK),
	m_sizer(NULL)
{
	m_window_flags = style;
	if (parent != NULL)
		parent->m_window_list.push_back(this);
}

wxncWindow::~wxncWindow(void)
{
	for (std::list<wxncWindow *>::const_iterator i = m_window_list.begin();
	     i != m_window_list.end(); ++i)
		delete *i;
	delete m_sizer;
}

bool wxncWindow::SetBackgroundColor(const wxncColor &color)
{
	m_bgcolor = color;
	return true;
}

bool wxncWindow::SetForegroundColor(const wxncColor &color)
{
	m_fgcolor = color;
	return true;
}

void wxncWindow::SetSizer(wxncBoxSizer *sizer, bool delete_old)
{
	if (delete_old)
		delete m_sizer;
	m_sizer = sizer;
}

void wxncWindow::SetSize(const wxSize &size)
{
	wxncPackable::SetSize(size);
	if (!(m_window_flags & wxBORDER_NONE))
		m_clientsize.DecBy(2, 2);
}

void wxncWindow::SetSize(int x, int y)
{
	SetSize(wxSize(x, y));
}

void wxncWindow::SetPosition(const wxPoint &pos)
{
	wxncPackable::SetPosition(pos);
	if (!(m_window_flags & wxBORDER_NONE)) {
		++m_clientpos.x;
		++m_clientpos.y;
	}
}

void wxncWindow::SetPosition(int x, int y)
{
	SetPosition(wxPoint(x, y));
}

void wxncWindow::Show(bool on)
{
}

void wxncWindow::RecalcSizes(void)
{
	if (m_sizer != NULL) {
		m_sizer->RecalcSizes();
		m_minsize = m_sizer->GetMinSize();
	} else {
		for (std::list<wxncWindow *>::const_iterator
		     w = m_window_list.begin(); w != m_window_list.end(); ++w)
		{
			(*w)->RecalcSizes();
			m_minsize.IncTo((*w)->GetMinSize());
		}
	}
	recalc_sizes_bh();
}

/**
 * Bottom half of RecalcSizes - add border, etc.
 * Called from e.g. wxncButton::RecalcSizes.
 */
void wxncWindow::recalc_sizes_bh(void)
{
	if (!(m_window_flags & wxBORDER_NONE))
		m_minsize.IncBy(2, 2);
}

void wxncWindow::Layout(void)
{
	if (m_sizer == NULL)
		return;

	m_sizer->SetPosition(m_clientpos);
	m_sizer->SetSize(m_sizer->GetMinSize());
	m_sizer->Layout();
}

void wxncWindow::Refresh(void)
{
	RecalcSizes();
	Layout();
	Update();
}

void wxncWindow::Update(void)
{
	/* Start positions */
	int &sx = m_pos.x;
	int &sy = m_pos.y;
	int ex = sx + m_size.x;
	int ey = sy + m_size.y;
	render_background(sx, sy, ex, ey);

	if (!(m_window_flags & wxBORDER_NONE))
		render_border(sx, sy, ex, ey);

	for (std::list<wxncWindow *>::const_iterator w = m_window_list.begin();
	     w != m_window_list.end(); ++w)
		(*w)->Update();
}

void wxncWindow::render_background(int sx, int sy, int ex, int ey)
{
	wxnc_clear_area(sx, sy, ex, ey,
		wxncColor::GetColor(m_fgcolor, m_bgcolor));
}

void wxncWindow::render_border(int sx, int sy, int ex, int ey)
{
	wxncRenderColor edge1, edge2;
	unsigned char edge1a, edge2a;

	if ((m_window_flags & wxBORDER_MASK) == 0 ||
	    (m_window_flags & wxBORDER_RAISED)) {
		edge1a = wxncColor::b_coherence_map[m_bgcolor.nc_index()][0];
		edge2a = wxncColor::b_coherence_map[m_bgcolor.nc_index()][1];
	} else if (m_window_flags & wxBORDER_SUNKEN) {
		edge1a = wxncColor::b_coherence_map[m_bgcolor.nc_index()][1];
		edge2a = wxncColor::b_coherence_map[m_bgcolor.nc_index()][0];
	} else {
		edge1a = edge2a = wxncColor::b_coherence_map[m_bgcolor.nc_index()][1];
	}

	edge1 = wxncColor::GetColor(wxncColor(edge1a), m_bgcolor);
	edge2 = wxncColor::GetColor(wxncColor(edge2a), m_bgcolor);

	int dx = ex - sx, dy = ey - sy; /* available space, for calculations */

	/* Always draw topleft corner, even if windows is size=(1x1) */
	attrset(edge1);
	mvaddch(sy, sx, ACS_ULCORNER);
	--dx; /* accont for having drawn it */
	--dy;

	if (dx >= 2 && dy >= 2) {
		mvhline(sy, sx + 1, ACS_HLINE, ex - sx - 2);
		mvvline(sy + 1, sx, ACS_VLINE, ey - sy - 2);
		mvaddch(sy, ex - 1, ACS_URCORNER);
		mvaddch(ey - 1, sx, ACS_LLCORNER);

		attrset(edge2);
		mvvline(sy + 1, ex - 1, ACS_VLINE, ey - sy - 2);
		mvhline(ey - 1, sx + 1, ACS_HLINE, ex - sx - 2);
		mvaddch(ey - 1, ex - 1, ACS_LRCORNER);

		/* bottom border, right border */
		--dx;
		--dy;
	}
}
